import sass from 'sass';
import round from 'round-to';
import rgbHex from 'rgb-hex';
import shortHexColor from 'shorten-css-hex';
import isPlainObject from 'is-plain-obj';
import parseColor from 'parse-color';
import parseUnit from 'parse-css-dimension';

/**
 * @typedef {import('../index').SassType} SassType
 * @typedef {import('../index').JsonValue} JsonValue
 * @typedef {import('../index').JsonObject} JsonObject
 * @typedef {import('../index').JsonArray} JsonArray
 */

/**
 * @param {SassType} value
 */
function getJsonValueFromSassValue(value) {
	let resolvedValue;
	if (value instanceof sass.types.List) {
		resolvedValue = listToArray(value);
	} else if (value instanceof sass.types.Map) {
		resolvedValue = mapToObject(value);
	} else if (value instanceof sass.types.Color) {
		/** @type {[number, number, number]} */
		const rgbValue = [value.getR(), value.getG(), value.getB()];
		const alphaValue = value.getA();
		if (alphaValue === 1) {
			resolvedValue = shortHexColor(`#${rgbHex.apply(null, rgbValue)}`);
		} else {
			resolvedValue = `rgba(${rgbValue.join(',')},${alphaValue})`;
		}
	} else if (value instanceof sass.types.Number) {
		if (value.getUnit() !== '') {
			resolvedValue = String(
				round(Number(value.getValue()), 0) + value.getUnit()
			);
		} else {
			resolvedValue = round(Number(value.getValue()), 0);
		}
	} else {
		try {
			if ('getValue' in value) {
				resolvedValue = value.getValue();
			} else {
				resolvedValue = null;
			}
		} catch (error) {
			resolvedValue = null;
		}
	}
	return resolvedValue;
}

/**
 * @param {sass.types.List} list
 */
function listToArray(list) {
	const length = list.getLength();
	/** @type {JsonArray} */
	const data = [];
	for (const index of Array.from({ length }).keys()) {
		const value = getJsonValueFromSassValue(list.getValue(index));
		data.push(value);
	}
	return data;
}

/**
 * @param {sass.types.Map} map
 */
function mapToObject(map) {
	const length = map.getLength();
	/** @type {JsonObject} */
	const data = {};
	for (const index of Array.from({ length }).keys()) {
		const key = String(map.getKey(index).getValue());
		const value = getJsonValueFromSassValue(map.getValue(index));
		data[key] = value;
	}
	return data;
}

/**
 * @typedef {import('../index').SassType} SassType
 * @typedef {import('../index').JsonValue} JsonValue
 * @typedef {import('../index').JsonObject} JsonObject
 * @typedef {import('../index').JsonArray} JsonArray
 */

const unitTypes = ['length', 'angle', 'resolution', 'frequency', 'time'];

/**
 * @param {string} value
 */
function isColor(value) {
	return typeof parseColor(value).rgba !== 'undefined';
}

/**
 * @param {string} value
 */
function parseValueToStringOrNumber(value) {
	let resolvedValue;
	try {
		const { value: parsedValue, unit, type } = parseUnit(value);
		if (unitTypes.includes(type)) {
			resolvedValue = new sass.types.Number(parsedValue, unit);
		} else if (type === 'percentage') {
			resolvedValue = new sass.types.Number(parsedValue, '%');
		} else {
			resolvedValue = new sass.types.String(value);
		}
	} catch (error) {
		resolvedValue = new sass.types.String(value);
	}
	return resolvedValue;
}

/**
 * @param {string} value
 */
function parseValueToColor(value) {
	const [r, g, b, a] = parseColor(value).rgba;
	return new sass.types.Color(r, g, b, a);
}

/**
 * @param {JsonValue} value
 */
function setJsonValueToSassValue(value) {
	let resolvedValue;
	if (Array.isArray(value)) {
		resolvedValue = arrayToList(value);
	} else if (isPlainObject(value)) {
		resolvedValue = objectToMap(value);
	} else if (isColor(String(value))) {
		resolvedValue = parseValueToColor(String(value));
	} else if (typeof value === 'string') {
		resolvedValue = parseValueToStringOrNumber(value);
	} else if (typeof value === 'number') {
		resolvedValue = new sass.types.Number(value);
	} else if (typeof value === 'boolean') {
		resolvedValue = value
			? sass.types.Boolean.TRUE
			: sass.types.Boolean.FALSE;
	} else {
		resolvedValue = sass.types.Null.NULL;
	}
	return resolvedValue;
}

/**
 * @param {JsonArray} array
 */
function arrayToList(array) {
	const length = array.length;
	const data = new sass.types.List(length);
	for (const [index, item] of array.entries()) {
		data.setValue(index, setJsonValueToSassValue(item));
	}
	return data;
}

/**
 * @param {JsonObject} object
 */
function objectToMap(object) {
	const length = Object.keys(object).length;
	const data = new sass.types.Map(length);
	for (const [index, [property, value = null]] of Object.entries(
		object
	).entries()) {
		data.setKey(index, setJsonValueToSassValue(property));
		data.setValue(index, setJsonValueToSassValue(value));
	}
	return data;
}

/**
 * @typedef {sass.types.Null|sass.types.Number|sass.types.String|sass.types.Boolean|sass.types.Color|sass.types.List|sass.types.Map} SassType
 */
/**
 * @typedef {JsonPrimitive | JsonObject | JsonArray} JsonValue
 * @typedef {JsonValue[]} JsonArray
 * @typedef {string | number | boolean | null} JsonPrimitive
 * @typedef {{[Key in string]?: JsonValue}} JsonObject
 */

/**
 * Encodes (`JSON.stringify`) data and returns Sass string. By default, string is quoted with single quotes so that it can be easily used in standard CSS values.
 *
 * @param {SassType}           value  Data to encode (stringify).
 * @param {sass.types.Boolean} quotes Should output string be quoted with single quotes.
 */
function encode(value, quotes) {
	const shouldQuote = quotes.getValue();
	let resolvedValue = JSON.stringify(getJsonValueFromSassValue(value));
	if (shouldQuote) {
		resolvedValue = `'${resolvedValue}'`;
	}
	return new sass.types.String(resolvedValue);
}

/**
 * Decodes (`JSON.parse`) string and returns one of available Sass types.
 *
 * @param {sass.types.String} value String to decode (parse).
 */
function decode(value) {
	/** @type {JsonValue?} */
	let resolvedValue = {};
	try {
		resolvedValue = JSON.parse(value.getValue());
	} catch (error) {
		resolvedValue = null;
	}
	return setJsonValueToSassValue(resolvedValue);
}

/** @type {{ 'json-encode($value, $quotes: true)': typeof encode, 'json-decode($value)': typeof decode }} */
const api = {
	'json-encode($value, $quotes: true)': encode,
	'json-decode($value)': decode
};

export { api as default };
//# sourceMappingURL=index.js.map
